#include "ehs/Encryption.h"
#include "ehs/PRNG.h"

namespace ehs
{
	void Encryption::Encrypt_64(const UInt_64 key, const UInt_64 size, Byte* const data)
	{
		PRNG_u64 rand(key);

		for (UInt_64 i = 0; i < size;)
		{
			if (i + sizeof(UInt_64) < size)
			{
				*((UInt_64*)&data[i]) ^= rand.Generate(0, (UInt_64)EHS_UINT_64_MAX);
				i += sizeof(UInt_64);
			}
			else if (i + sizeof(UInt_32) < size)
			{
				*((UInt_32*)&data[i]) ^= (UInt_32)rand.Generate(0, EHS_UINT_32_MAX);
				i += sizeof(UInt_32);
			}
			else if (i + sizeof(UInt_16) < size)
			{
				*((UInt_16*)&data[i]) ^= (UInt_16)rand.Generate(0, EHS_UINT_16_MAX);
				i += sizeof(UInt_16);
			}
			else
			{
				data[i] ^= (Byte) rand.Generate(0, EHS_UINT_8_MAX);
				++i;
			}
		}
	}

	void Encryption::Encrypt_64(const UInt_64 size, Byte* const data)
	{
		Encrypt_64(size, size, data);
	}

    void Encryption::Encrypt_64(Array<Byte> &data)
    {
		Encrypt_64(data.Size(), data);
    }

    void Encryption::Encrypt_64(Vector<Byte>& data)
    {
		Encrypt_64(data.Size(), data);
    }

	void Encryption::Encrypt_64(Serializer<UInt_64>& data)
	{
		Encrypt_64(data.Size(), data);
	}

	void Encryption::Encrypt_32(const UInt_64 key, const UInt_64 size, Byte* const data)
	{
		PRNG_u64 rand(key);

		for (UInt_64 i = 0; i < size;)
		{
			if (i + sizeof(UInt_32) < size)
			{
				*((UInt_32*)&data[i]) ^= rand.Generate(0, EHS_UINT_32_MAX);
				i += sizeof(UInt_32);
			}
			else if (i + sizeof(UInt_16) < size)
			{
				*((UInt_16*)&data[i]) ^= (UInt_16)rand.Generate(0, EHS_UINT_16_MAX);
				i += sizeof(UInt_16);
			}
			else
			{
				data[i] ^= (Byte)rand.Generate(0, EHS_UINT_8_MAX);
				++i;
			}
		}
	}

	void Encryption::Encrypt_32(const UInt_64 size, Byte* const data)
	{
		Encrypt_32(size, size, data);
	}

    void Encryption::Encrypt_32(Array<Byte>& data)
    {
		Encrypt_32(data.Size(), data);
    }

    void Encryption::Encrypt_32(Vector<Byte>& data)
    {
		Encrypt_32(data.Size(), data);
    }

	void Encryption::Encrypt_16(const UInt_64 key, const UInt_64 size, Byte* const data)
	{
		PRNG_u64 rand(key);

		for (UInt_64 i = 0; i < size;)
		{
			if (i + sizeof(UInt_16) < size)
			{
				*((UInt_16*)&data[i]) ^= (UInt_16)rand.Generate(0, EHS_UINT_16_MAX);
				i += sizeof(UInt_16);
			}
			else
			{
				data[i] ^= (Byte)rand.Generate(0, EHS_UINT_8_MAX);
				++i;
			}
		}
	}

	void Encryption::Encrypt_16(const UInt_64 size, Byte* const data)
	{
		Encrypt_16(size, size, data);
	}

	void Encryption::Encrypt_16(Array<Byte>& data)
	{
		Encrypt_16(data.Size(), data);
	}

	void Encryption::Encrypt_16(Vector<Byte>& data)
	{
		Encrypt_16(data.Size(), data);
	}

	void Encryption::Encrypt_8(const UInt_64 key, const UInt_64 size, Byte* const data)
	{
		PRNG_u64 rand(key);

		for (UInt_64 i = 0; i < size; ++i)
			data[i] ^= (Byte)rand.Generate(0, EHS_UINT_8_MAX);
	}

	void Encryption::Encrypt_8(const UInt_64 size, Byte* const data)
	{
		Encrypt_8(size, size, data);
	}

	void Encryption::Encrypt_8(Array<Byte>& data)
	{
		Encrypt_8(data.Size(), data);
	}

	void Encryption::Encrypt_8(Vector<Byte>& data)
	{
		Encrypt_8(data.Size(), data);
	}
}